@@ -35,7 +35,13 @@ DEFAULT_WWW_PATH = '/usr/share/pritunl/www'
 DEFAULT_DATA_PATH = '/var/lib/pritunl'
 DEFAULT_KEY_BITS = 4096
 DEFAULT_DH_PARAM_BITS = 1536
+DEFAULT_ORG_POOL_SIZE = 1
+DEFAULT_USER_POOL_SIZE = 8
+DEFAULT_SERVER_USER_POOL_SIZE = 4
+DEFAULT_SERVER_POOL_SIZE = 0
+DEFAULT_DH_PARAM_BITS_POOL = ['1536']
 DEFAULT_OTP_SECRET_LEN = 16
+DEFAULT_SERVER_LOG_LINES = 1024
 DEFAULT_PUBLIC_IP_SERVER = 'http://ip.pritunl.com/json'
 DEFAULT_NOTIFICATION_SERVER = 'http://ip.pritunl.com/notification'
 SUBSCRIPTION_SERVER = 'https://app.pritunl.com/subscription'
@@ -63,8 +69,10 @@ LOCALHOST_IP_TTL = 30
 AUTH_SIG_STRING_MAX_LEN = 10240
 AUTH_TIME_WINDOW = 300
 OTP_CACHE_TTL = 43200
+SUB_OFF = False
 IP_REGEX = r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}'
 SAFE_PUB_SUBNETS = {'50.203.224.0/24'}
+VALID_DH_PARAM_BITS = (1024, 1536, 2048, 3072, 4096)
 STATIC_FILE_EXTENSIONS = {
     '.css',
     '.eot',
@@ -90,6 +98,7 @@ REQS_DIR = 'reqs'
 KEYS_DIR = 'keys'
 CERTS_DIR = 'certs'
 USERS_DIR = 'users'
+DH_POOL_DIR = 'dh_param_pool'
 TEMP_DIR = 'temp'
 EMPTY_TEMP_DIR = 'empty_temp'
 OPENSSL_NAME = 'openssl.conf'
@@ -124,6 +133,8 @@ CA_CERT_ID = 'ca'
 CERT_CA = 'ca'
 CERT_SERVER = 'server'
 CERT_CLIENT = 'client'
+CERT_SERVER_POOL = 'server_pool'
+CERT_CLIENT_POOL = 'client_pool'
 
 ORGS_UPDATED = 'organizations_updated'
 USERS_UPDATED = 'users_updated'
@@ -407,6 +418,7 @@ import time
 import traceback
 
 VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
+api_key = '%s'
 auth_log_path = '%s'
 def log_write(line):
     with open(auth_log_path, 'a') as auth_log_file:
@@ -452,6 +464,7 @@ try:
     try:
         request = Request('%s://%s:%s' + \\
             '/server/%s/tls_verify')
+        request.add_header('API-Key', api_key)
         request.add_header('Content-Type', 'application/json')
         response = urlopen(request, json.dumps({
             'org_id': org,
@@ -487,6 +500,7 @@ import time
 import traceback
 
 VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
+api_key = '%s'
 auth_log_path = '%s'
 def log_write(line):
     with open(auth_log_path, 'a') as auth_log_file:
@@ -545,6 +559,7 @@ try:
     try:
         request = Request('%s://%s:%s' + \\
             '/server/%s/otp_verify')
+        request.add_header('API-Key', api_key)
         request.add_header('Content-Type', 'application/json')
         response = urlopen(request, json.dumps({
             'org_id': org,
@@ -582,6 +597,7 @@ import time
 import traceback
 
 VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
+api_key = '%s'
 auth_log_path = '%s'
 def log_write(line):
     with open(auth_log_path, 'a') as auth_log_file:
@@ -630,6 +646,7 @@ try:
     try:
         request = Request('%s://%s:%s' + \\
             '/server/%s/client_connect')
+        request.add_header('API-Key', api_key)
         request.add_header('Content-Type', 'application/json')
         response = urlopen(request, json.dumps({
             'org_id': org,
@@ -665,6 +682,7 @@ import time
 import traceback
 
 VALID_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789='
+api_key = '%s'
 auth_log_path = '%s'
 def log_write(line):
     with open(auth_log_path, 'a') as auth_log_file:
@@ -713,6 +731,7 @@ try:
     try:
         request = Request('%s://%s:%s' + \\
             '/server/%s/client_disconnect')
+        request.add_header('API-Key', api_key)
         request.add_header('Content-Type', 'application/json')
         response = urlopen(request, json.dumps({
             'org_id': org,
