@@ -12,6 +12,19 @@ class KeyLinkError(UserError):
     pass
 
 
+class EmailError(BaseError):
+    pass
+
+class EmailNotConfiguredError(BaseError):
+    pass
+
+class EmailFromInvalid(EmailError):
+    pass
+
+class EmailApiKeyInvalid(EmailError):
+    pass
+
+
 class ServerError(BaseError):
     pass
 
