@@ -19,27 +19,32 @@ logger = logging.getLogger(APP_NAME)
 
 class Organization(Config):
     str_options = {'name'}
+    bool_options = {'pool'}
     cached = True
     cache_prefix = 'org'
 
-    def __init__(self, id=None, name=None):
+    def __init__(self, id=None, name=None, pool=None):
         Config.__init__(self)
         self._last_prefix_count = None
 
         if id is None:
             self.id = uuid.uuid4().hex
             self.name = name
+            self.pool = pool
         else:
             self.id = id
 
-        self.path = os.path.join(app_server.data_path, ORGS_DIR, self.id)
-        self.set_path(os.path.join(self.path, 'ca.conf'))
-
         if id is None:
             self._initialize()
 
         self.ca_cert = User.get_user(self, id=CA_CERT_ID)
 
+    def __setattr__(self, name, value):
+        if name == 'id':
+            self.path = os.path.join(app_server.data_path, ORGS_DIR, value)
+            self.set_path(os.path.join(self.path, 'ca.conf'))
+        Config.__setattr__(self, name, value)
+
     def __getattr__(self, name):
         if name == 'otp_auth':
             for server in self.iter_servers():
@@ -48,6 +53,12 @@ class Organization(Config):
             return False
         elif name == 'user_count':
             return self._get_user_count()
+        elif name == 'client_pool_count':
+            self._cache_users()
+            return cache_db.set_length(self.get_cache_key('users_client_pool'))
+        elif name == 'server_pool_size':
+            self._cache_users()
+            return cache_db.set_length(self.get_cache_key('users_server_pool'))
         elif name == 'page_total':
             return int(cache_db.get(self.get_cache_key('users_page_total')))
         return Config.__getattr__(self, name)
@@ -79,12 +90,24 @@ class Organization(Config):
         utils.rmtree(os.path.join(self.path, 'indexed_certs'))
 
     def _initialize(self):
-        self._make_dirs()
+        pool_org_id = None
+        if not self.pool:
+            pool_org_id = cache_db.set_pop('orgs_pool')
+        if pool_org_id:
+            self.id = pool_org_id
+        else:
+            self._make_dirs()
         try:
-            self.ca_cert = User(self, type=CERT_CA)
-            cache_db.set_add('orgs', self.id)
+            if not pool_org_id:
+                self.ca_cert = User(self, type=CERT_CA)
+            if self.pool:
+                cache_db.set_add('orgs_pool', self.id)
+            else:
+                cache_db.set_add('orgs', self.id)
             self.commit()
-            LogEntry(message='Created new organization "%s".' % self.name)
+            cache_db.publish('users_pool', 'update')
+            if not self.pool:
+                LogEntry(message='Created new organization "%s".' % self.name)
         except:
             logger.exception('Failed to create organization. %r' % {
                 'org_id': self.id,
@@ -101,14 +124,19 @@ class Organization(Config):
         os.makedirs(os.path.join(self.path, TEMP_DIR))
 
     def clear_cache(self):
-        for user in self.iter_users():
-            user.clear_cache(org_data=False)
-        self.ca_cert.clear_cache(org_data=False)
         cache_db.set_remove('orgs', self.id)
         cache_db.list_remove('orgs_sorted', self.id)
-        cache_db.decrement('org_count')
+        if not self.pool:
+            cache_db.decrement('org_count')
+        for user in self.iter_users():
+            user.clear_cache(org_data=False)
+        for user in self.iter_users_pool():
+            user.clear_cache(org_data=False)
+        self.ca_cert.clear_cache(org_data=False)
         cache_db.remove(self.get_cache_key('users_cached'))
         cache_db.remove(self.get_cache_key('users'))
+        cache_db.remove(self.get_cache_key('users_client_pool'))
+        cache_db.remove(self.get_cache_key('users_server_pool'))
         cache_db.remove(self.get_cache_key('user_count'))
         cache_db.remove(self.get_cache_key('users_sorted'))
         cache_db.remove(self.get_cache_key('users_page_index'))
@@ -175,6 +203,8 @@ class Organization(Config):
     def _cache_users(self):
         if cache_db.get(self.get_cache_key('users_cached')) != 't':
             cache_db.remove(self.get_cache_key('users'))
+            cache_db.remove(self.get_cache_key('users_client_pool'))
+            cache_db.remove(self.get_cache_key('users_server_pool'))
             certs_path = os.path.join(self.path, CERTS_DIR)
             if os.path.isdir(certs_path):
                 for cert in os.listdir(certs_path):
@@ -184,8 +214,16 @@ class Organization(Config):
                     user = User.get_user(self, id=user_id)
                     if not user:
                         continue
-                    user._add_cache_trie_key()
-                    cache_db.set_add(self.get_cache_key('users'), user_id)
+                    if user.type == CERT_CLIENT_POOL:
+                        cache_db.set_add(
+                            self.get_cache_key('users_client_pool'), user_id)
+                    elif user.type == CERT_SERVER_POOL:
+                        cache_db.set_add(
+                            self.get_cache_key('users_server_pool'), user_id)
+                    else:
+                        user._add_cache_trie_key()
+                        cache_db.set_add(
+                            self.get_cache_key('users'), user_id)
             self.sort_users_cache()
             cache_db.set(self.get_cache_key('users_cached'), 't')
 
@@ -242,12 +280,12 @@ class Organization(Config):
                 user = User.get_user(self, id=user_id)
                 if not user:
                     continue
-                yield user
                 if prefix_limit and user_type == CERT_CLIENT:
                     user_count += 1
-                    if user_count >= prefix_limit:
+                    if user_count > prefix_limit:
                         search_more = True
                         break
+                yield user
             if prefix_limit and not search_more:
                 yield None
         else:
@@ -257,6 +295,13 @@ class Organization(Config):
                 if user:
                     yield user
 
+    def iter_users_pool(self):
+        for pool in ('users_client_pool', 'users_server_pool'):
+            for user_id in cache_db.set_iter(self.get_cache_key(pool)):
+                user = User.get_user(self, id=user_id)
+                if user:
+                    yield user
+
     def create_user_key_link(self, user_id):
         key_id = uuid.uuid4().hex
         key_id_key = 'key_token-%s' % key_id
@@ -337,6 +382,10 @@ class Organization(Config):
             if self.id in server.organizations:
                 yield server
 
+    def update_ip_pool(self):
+        from server import Server
+        Server.queue_update_ip_pool(org_id=self.id)
+
     def new_user(self, type, name=None, email=None):
         return User(self, name=name, email=email, type=type)
 
@@ -359,8 +408,9 @@ class Organization(Config):
 
     def commit(self):
         Config.commit(self)
-        self.sort_orgs_cache()
-        Event(type=ORGS_UPDATED)
+        if not self.pool:
+            self.sort_orgs_cache()
+            Event(type=ORGS_UPDATED)
 
     @classmethod
     def get_org(cls, id):
@@ -404,10 +454,16 @@ class Organization(Config):
     def _cache_orgs(cls):
         if cache_db.get('orgs_cached') != 't':
             cache_db.remove('orgs')
+            cache_db.remove('orgs_pool')
             path = os.path.join(app_server.data_path, ORGS_DIR)
             if os.path.isdir(path):
                 for org_id in os.listdir(path):
-                    cache_db.set_add('orgs', org_id)
+                    org = Organization.get_org(id=org_id)
+                    if org:
+                        if org.pool:
+                            cache_db.set_add('orgs_pool', org_id)
+                        else:
+                            cache_db.set_add('orgs', org_id)
             cls.sort_orgs_cache()
             cache_db.set('orgs_cached', 't')
 
@@ -420,6 +476,10 @@ class Organization(Config):
             org_count = int(cache_db.get('org_count'))
         return org_count
 
+    @classmethod
+    def get_org_pool_count(cls):
+        return cache_db.set_length('orgs_pool')
+
     @classmethod
     def iter_orgs(cls):
         cls._cache_orgs()
@@ -427,3 +487,47 @@ class Organization(Config):
             org = Organization.get_org(id=org_id)
             if org:
                 yield org
+
+    @classmethod
+    def iter_orgs_pool(cls):
+        cls._cache_orgs()
+        for org_id in cache_db.set_iter('orgs_pool'):
+            org = Organization.get_org(id=org_id)
+            if org:
+                yield org
+
+    @classmethod
+    def _sort_users_queue_thread(cls):
+        try:
+            for msg in cache_db.subscribe('sort_users_queue'):
+                if msg == 'update':
+                    while True:
+                        operation = cache_db.set_pop('sort_users_queues')
+                        if not operation:
+                            break
+                        org_id, update_ip_pool = operation.split(':')
+                        update_ip_pool = True if update_ip_pool == 't' \
+                            else False
+                        org = Organization.get_org(id=org_id)
+                        if org:
+                            org.sort_users_cache()
+                            if update_ip_pool:
+                                org.update_ip_pool()
+                            Event(type=ORGS_UPDATED)
+                            Event(type=USERS_UPDATED, resource_id=org.id)
+                            Event(type=SERVERS_UPDATED)
+        except:
+            logger.exception('Exception in sort users queue.')
+            cls._ip_pool_queue_thread()
+
+    @classmethod
+    def setup_sort_users_queue(cls):
+        thread = threading.Thread(target=cls._sort_users_queue_thread)
+        thread.daemon = True
+        thread.start()
+
+    def queue_sort_users_cache(self, update_ip_pool=False):
+        # Remove overlap sort users updates
+        cache_db.set_add('sort_users_queues',
+            '%s:%s' % (self.id, 't' if update_ip_pool else 'f'))
+        cache_db.publish('sort_users_queue', 'update')
