@@ -222,11 +222,14 @@ class Server(Config):
             pool['network'] = self.network
             ip_pool_file.write(json.dumps(pool))
 
-    def update_ip_pool(self):
+    def update_ip_pool(self, reset=False, silent=False):
         cache_key = self.get_cache_key('ip_pool')
         set_cache_key = self.get_cache_key('ip_pool_set')
         cache_db.lock_acquire(cache_key)
         try:
+            if reset:
+                cache_db.remove(cache_key)
+                cache_db.remove(set_cache_key)
             ip_pool = ipaddress.IPv4Network(self.network).iterhosts()
             ip_pool.next()
 
@@ -264,8 +267,9 @@ class Server(Config):
                 pass
             finally:
                 self._commit_ip_pool()
-                for org in self.iter_orgs():
-                    Event(type=USERS_UPDATED, resource_id=org.id)
+                if not silent:
+                    for org in self.iter_orgs():
+                        Event(type=USERS_UPDATED, resource_id=org.id)
         finally:
             cache_db.lock_release(cache_key)
 
@@ -330,11 +334,9 @@ class Server(Config):
     def commit(self):
         if self._rebuild_dh_params:
             self._generate_dh_param()
-        if self._reset_ip_pool:
-            cache_db.remove(self.get_cache_key('ip_pool'))
-            cache_db.remove(self.get_cache_key('ip_pool_set'))
-            self.update_ip_pool()
         Config.commit(self)
+        if self._reset_ip_pool:
+            self.queue_update_ip_pool(server_id=self.id, reset=True)
         self.sort_servers_cache()
         Event(type=SERVERS_UPDATED)
 
@@ -376,7 +378,7 @@ class Server(Config):
             return org
         self.organizations.append(org.id)
         self.commit()
-        self.update_ip_pool()
+        self.queue_update_ip_pool(server_id=self.id)
         Event(type=SERVERS_UPDATED)
         Event(type=SERVER_ORGS_UPDATED, resource_id=self.id)
         Event(type=USERS_UPDATED, resource_id=org_id)
@@ -417,7 +419,7 @@ class Server(Config):
             self._remove_primary_user()
         self.organizations.remove(org_id)
         self.commit()
-        self.update_ip_pool()
+        self.queue_update_ip_pool(server_id=self.id)
         Event(type=SERVERS_UPDATED)
         Event(type=SERVER_ORGS_UPDATED, resource_id=self.id)
         Event(type=USERS_UPDATED, resource_id=org_id)
@@ -466,13 +468,19 @@ class Server(Config):
             'server_id': self.id,
         })
         self._rebuild_dh_params = False
-        args = [
-            'openssl', 'dhparam',
-            '-out', self.dh_param_path,
-            str(self.dh_param_bits)
-        ]
-        subprocess.check_call(args, stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
+
+        dh_pool_path = cache_db.set_pop('dh_pool_%s' % self.dh_param_bits)
+        if dh_pool_path:
+            cache_db.publish('servers_pool', 'update')
+            os.rename(dh_pool_path, self.dh_param_path)
+        else:
+            args = [
+                'openssl', 'dhparam',
+                '-out', self.dh_param_path,
+                str(self.dh_param_bits),
+            ]
+            subprocess.check_call(args, stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE)
 
     def _parse_network(self, network):
         network_split = network.split('/')
@@ -509,6 +517,7 @@ class Server(Config):
             with open(script_path, 'w') as script_file:
                 os.chmod(script_path, 0755)
                 script_file.write(script % (
+                    app_server.local_api_key,
                     self.auth_log_path,
                     app_server.web_protocol,
                     auth_host,
@@ -940,7 +949,13 @@ class Server(Config):
         self._event_delay(type=SERVER_OUTPUT_UPDATED, resource_id=self.id)
 
     def push_output(self, output):
+        if not app_server.server_log_lines:
+            return
         cache_db.list_rpush(self.get_cache_key('output'), output.rstrip('\n'))
+        clear_lines = cache_db.list_length(self.get_cache_key('output')) - \
+            app_server.server_log_lines
+        for _ in xrange(clear_lines):
+            cache_db.list_lpop(self.get_cache_key('output'))
         self._event_delay(type=SERVER_OUTPUT_UPDATED, resource_id=self.id)
 
     def get_bandwidth(self, period=None):
@@ -1287,3 +1302,53 @@ class Server(Config):
             server = cls.get_server(id=server_id, type=server_type)
             if server:
                 yield server
+
+    @classmethod
+    def _ip_pool_queue_thread(cls):
+        try:
+            for msg in cache_db.subscribe('ip_pool_queue'):
+                if msg == 'update':
+                    while True:
+                        operation = cache_db.set_pop('ip_pool_queues')
+                        if not operation:
+                            break
+                        org_id, server_id, reset = operation.split(':')
+                        reset = True if reset == 't' else False
+                        if org_id:
+                            org = Organization.get_org(id=org_id)
+                            if org:
+                                for server in org.iter_servers():
+                                    try:
+                                        server.update_ip_pool(reset=reset)
+                                    except:
+                                        logger.exception(
+                                            'Update ip pool error.')
+                        if server_id:
+                            server = Server.get_server(id=server_id)
+                            if server:
+                                try:
+                                    server.update_ip_pool(reset=reset)
+                                except:
+                                    logger.exception('Update ip pool error.')
+                        if not org_id and not server_id:
+                            for server in cls.iter_servers():
+                                try:
+                                    server.update_ip_pool(reset=reset)
+                                except:
+                                    logger.exception('Update ip pool error.')
+        except:
+            logger.exception('Exception in ip pool queue.')
+            cls._ip_pool_queue_thread()
+
+    @classmethod
+    def setup_ip_pool_queue(cls):
+        thread = threading.Thread(target=cls._ip_pool_queue_thread)
+        thread.daemon = True
+        thread.start()
+
+    @classmethod
+    def queue_update_ip_pool(cls, org_id='', server_id='', reset=False):
+        # Remove overlap ip pool updates
+        cache_db.set_add('ip_pool_queues',
+            '%s:%s:%s' % (org_id, server_id, 't' if reset else 'f'))
+        cache_db.publish('ip_pool_queue', 'update')
