@@ -50,26 +50,30 @@ class User(Config):
         else:
             self.id = id
 
-        self.reqs_path = os.path.join(self.org.path, REQS_DIR,
-            '%s.csr' % self.id)
-        self.key_path = os.path.join(self.org.path, KEYS_DIR,
-            '%s.key' % self.id)
-        self.cert_path = os.path.join(self.org.path, CERTS_DIR,
-            '%s.crt' % self.id)
-        self.temp_key_archive_path = os.path.join(self.org.path,
-            TEMP_DIR, '%s_%s.tar' % (self.id, uuid.uuid4().hex))
-        self.set_path(os.path.join(self.org.path, USERS_DIR,
-            '%s.conf' % self.id))
-
-        self.temp_path = os.path.join(self.org.path, TEMP_DIR, self.id)
-        self.ssl_conf_path = os.path.join(self.temp_path, OPENSSL_NAME)
-        self.index_path = os.path.join(self.temp_path, INDEX_NAME)
-        self.index_attr_path = os.path.join(self.temp_path, INDEX_ATTR_NAME)
-        self.serial_path = os.path.join(self.temp_path, SERIAL_NAME)
-
         if id is None:
             self._initialize()
 
+    def __setattr__(self, name, value):
+        if name == 'id':
+            self.reqs_path = os.path.join(self.org.path, REQS_DIR,
+                '%s.csr' % value)
+            self.key_path = os.path.join(self.org.path, KEYS_DIR,
+                '%s.key' % value)
+            self.cert_path = os.path.join(self.org.path, CERTS_DIR,
+                '%s.crt' % value)
+            self.temp_key_archive_path = os.path.join(self.org.path,
+                TEMP_DIR, '%s_%s.tar' % (value, uuid.uuid4().hex))
+            self.set_path(os.path.join(self.org.path, USERS_DIR,
+                '%s.conf' % value))
+
+            self.temp_path = os.path.join(self.org.path, TEMP_DIR, value)
+            self.ssl_conf_path = os.path.join(self.temp_path, OPENSSL_NAME)
+            self.index_path = os.path.join(self.temp_path, INDEX_NAME)
+            self.index_attr_path = os.path.join(
+                self.temp_path, INDEX_ATTR_NAME)
+            self.serial_path = os.path.join(self.temp_path, SERIAL_NAME)
+        Config.__setattr__(self, name, value)
+
     def dict(self):
         return {
             'id': self.id,
@@ -99,27 +103,39 @@ class User(Config):
             self.commit()
 
     def _initialize(self):
-        self._setup_openssl()
-        self._cert_request()
-        self._generate_otp_secret()
-        self.commit()
-        self._cert_create()
-        self._clean_openssl()
+        pool_user_id = None
+        if self.type == CERT_CLIENT:
+            pool_user_id = cache_db.set_pop(
+                self.org.get_cache_key('users_client_pool'))
+        elif self.type == CERT_SERVER:
+            pool_user_id = cache_db.set_pop(
+                self.org.get_cache_key('users_server_pool'))
 
-        if self.type != CERT_CA:
+        if pool_user_id:
+            cache_db.publish('users_pool', 'update')
+            self.id = pool_user_id
+            self.commit()
+        else:
+            self._setup_openssl()
+            self._cert_request()
+            self._generate_otp_secret()
+            self.commit()
+            self._cert_create()
+            self._clean_openssl()
+
+        if self.type == CERT_CLIENT_POOL:
+            cache_db.set_add(
+                self.org.get_cache_key('users_client_pool'), self.id)
+        elif self.type == CERT_SERVER_POOL:
+            cache_db.set_add(
+                self.org.get_cache_key('users_server_pool'), self.id)
+        elif self.type != CERT_CA:
             cache_db.set_add(self.org.get_cache_key('users'), self.id)
             self._add_cache_trie_key()
 
-        self.org.sort_users_cache()
-        if self.type == CERT_CLIENT:
-            LogEntry(message='Created new user "%s".' % self.name)
-
-            for server in self.org.iter_servers():
-                server.update_ip_pool()
-
-        Event(type=ORGS_UPDATED)
-        Event(type=USERS_UPDATED, resource_id=self.org.id)
-        Event(type=SERVERS_UPDATED)
+            self.org.queue_sort_users_cache(update_ip_pool=True)
+            if self.type == CERT_CLIENT:
+                LogEntry(message='Created new user "%s".' % self.name)
 
     def _add_cache_trie_key(self):
         users_trie = CacheTrie(self.org.get_cache_key('users_trie'))
@@ -128,7 +144,7 @@ class User(Config):
 
     def _remove_cache_trie_key(self):
         users_trie = CacheTrie(self.org.get_cache_key('users_trie'))
-        users_trie.remove_key(self.name, '%s-%s-%s' % (
+        users_trie.remove_key_terms(self.name, '%s-%s-%s' % (
             self.id, self.type, self.name))
 
     def _setup_openssl(self):
@@ -163,7 +179,7 @@ class User(Config):
                 '-config', self.ssl_conf_path,
                 '-out', self.reqs_path,
                 '-keyout', self.key_path,
-                '-reqexts', '%s_req_ext' % self.type,
+                '-reqexts', '%s_req_ext' % self.type.replace('_pool', ''),
             ]
             subprocess.check_call(args, stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE)
@@ -184,7 +200,7 @@ class User(Config):
                 '-config', self.ssl_conf_path,
                 '-in', self.reqs_path,
                 '-out', self.cert_path,
-                '-extensions', '%s_ext' % self.type,
+                '-extensions', '%s_ext' % self.type.replace('_pool', ''),
             ]
             subprocess.check_call(args, stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE)
@@ -414,9 +430,14 @@ class User(Config):
                 'From': email_from,
                 'To': self.email,
                 'Subject': 'Pritunl VPN Key',
-                'TextBody':  'Your vpn key and configuration can be ' +
-                    'downloaded from the temporary link below:\n\n' +
-                    key_link_domain + key_link['view_url'],
+                'TextBody':  'Your vpn key can be downloaded from the ' +
+                    'temporary link below. You may also directly import ' +
+                    'your keys in the Pritunl client using the temporary ' +
+                    'URI link.\n\n' +
+                    'Key Link: ' + key_link_domain + key_link['view_url'] +
+                    '\nURI Key Link: ' +
+                    key_link_domain.replace('http', 'pt', 1) +
+                    key_link['uri_url'],
             },
         )
         response = response.json()
@@ -461,7 +482,7 @@ class User(Config):
                 cache_db.set_remove(self.org.get_cache_key('users'), self.id)
                 cache_db.list_remove(self.org.get_cache_key('users_sorted'),
                     self.id)
-        if self.type != CERT_CA:
+        if self.type in (CERT_CLIENT, CERT_SERVER):
             self._remove_cache_trie_key()
         cache_db.remove(self.get_cache_key('otp'))
         cache_db.remove(self.get_cache_key('otp_cache'))
@@ -472,13 +493,14 @@ class User(Config):
         self.name = name
         self.commit()
         self._add_cache_trie_key()
-        self.org.sort_users_cache()
+        self.org.queue_sort_users_cache()
         Event(type=USERS_UPDATED, resource_id=self.org.id)
 
     def remove(self):
         self.clear_cache()
         name = self.name
         type = self.type
+        self.org.queue_sort_users_cache(update_ip_pool=(type == CERT_CLIENT))
 
         try:
             os.remove(self.reqs_path)
@@ -522,10 +544,6 @@ class User(Config):
 
         self._clean_openssl()
 
-        if type == CERT_CLIENT:
-            for server in self.org.iter_servers():
-                server.update_ip_pool()
-
         Event(type=ORGS_UPDATED)
         Event(type=USERS_UPDATED, resource_id=self.org.id)
         if type == CERT_CLIENT:
@@ -536,10 +554,12 @@ class User(Config):
         user = User(org, id=id)
         try:
             user.load()
-        except IOError:
-            logger.exception('Failed to load user conf. %r' % {
-                'org_id': org.id,
-                'user_id': id,
-            })
+        except IOError as exception:
+            # File not found will happen when cached user list update is queued
+            if exception.errno != 2:
+                logger.exception('Failed to load user conf. %r' % {
+                    'org_id': org.id,
+                    'user_id': id,
+                })
             return
         return user
