@@ -8,7 +8,8 @@ define([
     defaults: {
       'id': null,
       'key_url': null,
-      'view_url': null
+      'view_url': null,
+      'uri_url': null
     },
     url: function() {
       return '/key/' + this.get('organization') + '/' + this.get('user');
