@@ -10,11 +10,13 @@ define([
       'organization': null,
       'organization_name': null,
       'name': null,
+      'email': null,
       'type': null,
       'status': null,
       'otp_auth': null,
       'otp_secret': null,
-      'servers': null
+      'servers': null,
+      'disabled': null
     },
     url: function() {
       var url = '/user/' + this.get('organization');
