@@ -16,11 +16,14 @@ define([
       'keyup input': 'onInputChange',
       'paste input': 'onInputChange',
       'input input': 'onInputChange',
+      'keypress .modal': 'onKeypress',
+      'shown.bs.modal .modal': 'onModalShown',
       'propertychange input': 'onInputChange'
     },
     title: '',
     okText: 'Ok',
     cancelText: 'Cancel',
+    enterOk: true,
     safeClose: false,
     body: function() {
       return '';
@@ -151,9 +154,17 @@ define([
         this.trigger('applied');
       }
       this.clearAlert();
-      this.clearLoading();
       this.$('.modal').modal('hide');
     },
+    onKeypress: function(evt) {
+      if (evt.keyCode === 13 && this.enterOk &&
+          !this.$('.ok').attr('disabled')) {
+        this.onOk();
+      }
+    },
+    onModalShown: function() {
+      this.$('[autofocus]').focus();
+    },
     onOk: function() {
       this.close();
     },
