@@ -11,6 +11,7 @@ define([
     okText: 'Add',
     loadingMsg: 'Adding server, this will take several minutes...',
     errorMsg: 'Failed to add server, server error occurred.',
+    safeClose: true,
     initialize: function(options) {
       this.usedNetworks = options.usedNetworks;
       this.usedPorts = options.usedPorts;
@@ -24,6 +25,7 @@ define([
         'port': this._get_free_port(),
         'protocol': 'udp',
         'dh_param_bits': 1536,
+        'mode': 'all_traffic',
         'public_address': options.publicIp || '',
         'local_networks': [],
         'dns_servers': ['8.8.8.8']
