@@ -22,8 +22,6 @@ define([
     listErrorMsg: 'Failed to load servers, server error occurred.',
     events: {
       'click .servers-add-server': 'onAddServer',
-      'mouseover .servers-add-server': 'onOverServer',
-      'mouseout .servers-add-server': 'onOutServer',
       'click .servers-attach-org': 'onAttachOrg'
     },
     initialize: function() {
@@ -95,22 +93,8 @@ define([
         }.bind(this)
       });
     },
-    onAddServer: function(evt) {
-      if ($(evt.target).text() === 'Add Node Server') {
-        this._addServer('node_server');
-      }
-      else {
-        this._addServer('server');
-      }
-      $(evt.target).text('Add Server');
-    },
-    onOverServer: function(evt) {
-      if (evt.ctrlKey || evt.shiftKey) {
-        $(evt.target).text('Add Node Server');
-      }
-    },
-    onOutServer: function(evt) {
-      $(evt.target).text('Add Server');
+    onAddServer: function() {
+      this._addServer('server');
     },
     onAttachOrg: function() {
       if (this.orgs.models.length) {
